@@ -1,38 +1,33 @@
-# Copyright 2013 Hynek Schlawack
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# This file is dual licensed under the terms of the Apache License, Version
+# 2.0, and the MIT License.  See the LICENSE file in the root of this
+# repository for complete details.
 
 from __future__ import absolute_import, division, print_function
 
+import platform
+import sys
 import warnings
 
+from collections import OrderedDict
+
 import pytest
 
-from pretend import call_recorder, call, stub
+from pretend import call, call_recorder, stub
+from six import PY3
+
+import structlog
 
 from structlog._base import BoundLoggerBase
-from structlog._compat import PY3
 from structlog._config import (
-    BoundLoggerLazyProxy,
-    _CONFIG,
     _BUILTIN_DEFAULT_CONTEXT_CLASS,
-    _BUILTIN_DEFAULT_PROCESSORS,
     _BUILTIN_DEFAULT_LOGGER_FACTORY,
+    _BUILTIN_DEFAULT_PROCESSORS,
     _BUILTIN_DEFAULT_WRAPPER_CLASS,
+    _CONFIG,
+    BoundLoggerLazyProxy,
     configure,
     configure_once,
     get_logger,
-    reset_defaults,
     wrap_logger,
 )
 
@@ -51,22 +46,56 @@ class Wrapper(BoundLoggerBase):
     """
 
 
+def test_default_context_class():
+    """
+    Default context class is dict on Python 3.6+ and PyPy, OrderedDict
+    otherwise.
+    """
+    if platform.python_implementation() == "PyPy" or sys.version_info[:2] >= (
+        3,
+        6,
+    ):
+        cls = dict
+    else:
+        cls = OrderedDict
+
+    assert cls is _BUILTIN_DEFAULT_CONTEXT_CLASS
+
+
 class TestConfigure(object):
     def teardown_method(self, method):
-        reset_defaults()
+        structlog.reset_defaults()
+
+    def test_get_config_is_configured(self):
+        """
+        Return value of structlog.get_config() works as input for
+        structlog.configure(). is_configured() reflects the state of
+        configuration.
+        """
+        assert False is structlog.is_configured()
+
+        structlog.configure(**structlog.get_config())
+
+        assert True is structlog.is_configured()
+
+        structlog.reset_defaults()
+
+        assert False is structlog.is_configured()
 
     def test_configure_all(self, proxy):
         x = stub()
         configure(processors=[x], context_class=dict)
         b = proxy.bind()
+
         assert [x] == b._processors
         assert dict is b._context.__class__
 
     def test_reset(self, proxy):
         x = stub()
         configure(processors=[x], context_class=dict, wrapper_class=Wrapper)
-        reset_defaults()
+        structlog.reset_defaults()
         b = proxy.bind()
+
         assert [x] != b._processors
         assert _BUILTIN_DEFAULT_PROCESSORS == b._processors
         assert isinstance(b, _BUILTIN_DEFAULT_WRAPPER_CLASS)
@@ -92,11 +121,6 @@ class TestConfigure(object):
         configure()
         assert True is _CONFIG.is_configured
 
-    def test_rest_resets_is_configured(self):
-        configure()
-        reset_defaults()
-        assert False is _CONFIG.is_configured
-
     def test_configures_logger_factory(self):
         def f():
             pass
@@ -107,20 +131,22 @@ class TestConfigure(object):
 
 class TestBoundLoggerLazyProxy(object):
     def teardown_method(self, method):
-        reset_defaults()
+        structlog.reset_defaults()
 
     def test_repr(self):
         p = BoundLoggerLazyProxy(
-            None, processors=[1, 2, 3], context_class=dict,
-            initial_values={'foo': 42}, logger_factory_args=(4, 5),
+            None,
+            processors=[1, 2, 3],
+            context_class=dict,
+            initial_values={"foo": 42},
+            logger_factory_args=(4, 5),
         )
         assert (
             "<BoundLoggerLazyProxy(logger=None, wrapper_class=None, "
             "processors=[1, 2, 3], "
             "context_class=<%s 'dict'>, "
             "initial_values={'foo': 42}, "
-            "logger_factory_args=(4, 5))>"
-            % ('class' if PY3 else 'type',)
+            "logger_factory_args=(4, 5))>" % ("class" if PY3 else "type",)
         ) == repr(p)
 
     def test_returns_bound_logger_on_bind(self, proxy):
@@ -130,8 +156,9 @@ class TestBoundLoggerLazyProxy(object):
         assert isinstance(proxy.new(), BoundLoggerBase)
 
     def test_prefers_args_over_config(self):
-        p = BoundLoggerLazyProxy(None, processors=[1, 2, 3],
-                                 context_class=dict)
+        p = BoundLoggerLazyProxy(
+            None, processors=[1, 2, 3], context_class=dict
+        )
         b = p.bind()
         assert isinstance(b._context, dict)
         assert [1, 2, 3] == b._processors
@@ -142,6 +169,7 @@ class TestBoundLoggerLazyProxy(object):
 
             def update(self, *args, **kw):
                 pass
+
         configure(processors=[4, 5, 6], context_class=Class)
         b = p.bind()
         assert not isinstance(b._context, Class)
@@ -153,20 +181,20 @@ class TestBoundLoggerLazyProxy(object):
         assert _CONFIG.default_processors == b._processors
 
     def test_bind_honors_initial_values(self):
-        p = BoundLoggerLazyProxy(None, initial_values={'a': 1, 'b': 2})
+        p = BoundLoggerLazyProxy(None, initial_values={"a": 1, "b": 2})
         b = p.bind()
-        assert {'a': 1, 'b': 2} == b._context
+        assert {"a": 1, "b": 2} == b._context
         b = p.bind(c=3)
-        assert {'a': 1, 'b': 2, 'c': 3} == b._context
+        assert {"a": 1, "b": 2, "c": 3} == b._context
 
     def test_bind_binds_new_values(self, proxy):
         b = proxy.bind(c=3)
-        assert {'c': 3} == b._context
+        assert {"c": 3} == b._context
 
     def test_unbind_unbinds_from_initial_values(self):
-        p = BoundLoggerLazyProxy(None, initial_values={'a': 1, 'b': 2})
-        b = p.unbind('a')
-        assert {'b': 2} == b._context
+        p = BoundLoggerLazyProxy(None, initial_values={"a": 1, "b": 2})
+        b = p.unbind("a")
+        assert {"b": 2} == b._context
 
     def test_honors_wrapper_class(self):
         p = BoundLoggerLazyProxy(None, wrapper_class=Wrapper)
@@ -179,16 +207,16 @@ class TestBoundLoggerLazyProxy(object):
         assert isinstance(b, Wrapper)
 
     def test_new_binds_only_initial_values_impolicit_ctx_class(self, proxy):
-        proxy = BoundLoggerLazyProxy(None, initial_values={'a': 1, 'b': 2})
+        proxy = BoundLoggerLazyProxy(None, initial_values={"a": 1, "b": 2})
         b = proxy.new(foo=42)
-        assert {'a': 1, 'b': 2, 'foo': 42} == b._context
+        assert {"a": 1, "b": 2, "foo": 42} == b._context
 
     def test_new_binds_only_initial_values_explicit_ctx_class(self, proxy):
-        proxy = BoundLoggerLazyProxy(None,
-                                     initial_values={'a': 1, 'b': 2},
-                                     context_class=dict)
+        proxy = BoundLoggerLazyProxy(
+            None, initial_values={"a": 1, "b": 2}, context_class=dict
+        )
         b = proxy.new(foo=42)
-        assert {'a': 1, 'b': 2, 'foo': 42} == b._context
+        assert {"a": 1, "b": 2, "foo": 42} == b._context
 
     def test_rebinds_bind_method(self, proxy):
         """
@@ -222,10 +250,44 @@ class TestBoundLoggerLazyProxy(object):
         proxy.bind()
         assert bind != proxy.bind
 
+    def test_bind_doesnt_cache_logger(self):
+        """
+        Calling configure() changes BoundLoggerLazyProxys immediately.
+        Previous uses of the BoundLoggerLazyProxy don't interfere.
+        """
+
+        class F(object):
+            "New logger factory with a new attribute"
+
+            def a(self, *args):
+                return 5
+
+        proxy = BoundLoggerLazyProxy(None)
+        proxy.bind()
+        configure(logger_factory=F)
+        new_b = proxy.bind()
+
+        assert new_b.a("test") == 5
+
+    def test_emphemeral(self):
+        """
+        Calling an unknown method proxy creates a new wrapped bound logger
+        first.
+        """
+
+        class Foo(BoundLoggerBase):
+            def foo(self):
+                return 42
+
+        proxy = BoundLoggerLazyProxy(
+            None, wrapper_class=Foo, cache_logger_on_first_use=False
+        )
+        assert 42 == proxy.foo()
+
 
 class TestFunctions(object):
     def teardown_method(self, method):
-        reset_defaults()
+        structlog.reset_defaults()
 
     def test_wrap_passes_args(self):
         logger = object()
@@ -234,6 +296,16 @@ class TestFunctions(object):
         assert [1, 2, 3] == p._processors
         assert dict is p._context_class
 
+    def test_empty_processors(self):
+        """
+        An empty list is a valid value for processors so it must be preserved.
+        """
+        # We need to do a bind such that we get an actual logger and not just
+        # a lazy proxy.
+        logger = wrap_logger(object(), processors=[]).new()
+
+        assert [] == logger._processors
+
     def test_wrap_returns_proxy(self):
         assert isinstance(wrap_logger(None), BoundLoggerLazyProxy)
 
@@ -245,12 +317,13 @@ class TestFunctions(object):
             configure_once()
         assert 1 == len(warns)
         assert RuntimeWarning == warns[0].category
-        assert 'Repeated configuration attempted.' == warns[0].message.args[0]
+        assert "Repeated configuration attempted." == warns[0].message.args[0]
 
     def test_get_logger_configures_according_to_config(self):
         b = get_logger().bind()
-        assert isinstance(b._logger,
-                          _BUILTIN_DEFAULT_LOGGER_FACTORY().__class__)
+        assert isinstance(
+            b._logger, _BUILTIN_DEFAULT_LOGGER_FACTORY().__class__
+        )
         assert _BUILTIN_DEFAULT_PROCESSORS == b._processors
         assert isinstance(b, _BUILTIN_DEFAULT_WRAPPER_CLASS)
         assert _BUILTIN_DEFAULT_CONTEXT_CLASS == b._context.__class__
@@ -262,5 +335,5 @@ class TestFunctions(object):
         """
         factory = call_recorder(lambda *args: object())
         configure(logger_factory=factory)
-        get_logger('test').bind(x=42)
-        assert [call('test')] == factory.calls
+        get_logger("test").bind(x=42)
+        assert [call("test")] == factory.calls
