@@ -1,54 +1,86 @@
-# Copyright 2013 Hynek Schlawack
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# This file is dual licensed under the terms of the Apache License, Version
+# 2.0, and the MIT License.  See the LICENSE file in the root of this
+# repository for complete details.
 
 from __future__ import absolute_import, division, print_function
 
 import sys
 
+import pytest
+
+from six.moves import cStringIO as StringIO
+
 from structlog._loggers import (
+    WRITE_LOCKS,
     PrintLogger,
     PrintLoggerFactory,
     ReturnLogger,
     ReturnLoggerFactory,
 )
+from structlog.stdlib import _NAME_TO_LEVEL
 
 
 def test_return_logger():
-    obj = ['hello']
+    obj = ["hello"]
     assert obj is ReturnLogger().msg(obj)
 
 
+STDLIB_MSG_METHODS = [m for m in _NAME_TO_LEVEL if m != "notset"]
+
+
 class TestPrintLogger(object):
     def test_prints_to_stdout_by_default(self, capsys):
-        PrintLogger().msg('hello')
+        """
+        Instantiating without arguments gives conveniently a logger to standard
+        out.
+        """
+        PrintLogger().msg("hello")
+
         out, err = capsys.readouterr()
-        assert 'hello\n' == out
-        assert '' == err
+        assert "hello\n" == out
+        assert "" == err
 
     def test_prints_to_correct_file(self, tmpdir, capsys):
-        f = tmpdir.join('test.log')
-        fo = f.open('w')
-        PrintLogger(fo).msg('hello')
+        """
+        Supplied files are respected.
+        """
+        f = tmpdir.join("test.log")
+        fo = f.open("w")
+        PrintLogger(fo).msg("hello")
         out, err = capsys.readouterr()
-        assert '' == out == err
+
+        assert "" == out == err
         fo.close()
-        assert 'hello\n' == f.read()
+        assert "hello\n" == f.read()
 
     def test_repr(self):
-        assert repr(PrintLogger()).startswith(
-            "<PrintLogger(file="
-        )
+        """
+        __repr__ makes sense.
+        """
+        assert repr(PrintLogger()).startswith("<PrintLogger(file=")
+
+    def test_lock(self):
+        """
+        Creating a logger adds a lock to WRITE_LOCKS.
+        """
+        sio = StringIO()
+
+        assert sio not in WRITE_LOCKS
+
+        PrintLogger(sio)
+
+        assert sio in WRITE_LOCKS
+
+    @pytest.mark.parametrize("method", STDLIB_MSG_METHODS)
+    def test_stdlib_methods_support(self, method):
+        """
+        PrintLogger implements methods of stdlib loggers.
+        """
+        sio = StringIO()
+
+        getattr(PrintLogger(sio), method)("hello")
+
+        assert "hello" in sio.getvalue()
 
 
 class TestPrintLoggerFactory(object):
@@ -57,14 +89,16 @@ class TestPrintLoggerFactory(object):
         Due to doctest weirdness, we must not re-use PrintLoggers.
         """
         f = PrintLoggerFactory()
+
         assert f() is not f()
 
     def test_passes_file(self):
         """
         If a file is passed to the factory, it get passed on to the logger.
         """
-        l = PrintLoggerFactory(sys.stderr)()
-        assert sys.stderr is l._file
+        pl = PrintLoggerFactory(sys.stderr)()
+
+        assert sys.stderr is pl._file
 
     def test_ignores_args(self):
         """
@@ -74,9 +108,21 @@ class TestPrintLoggerFactory(object):
         PrintLoggerFactory()(1, 2, 3)
 
 
+class ReturnLoggerTest(object):
+    @pytest.mark.parametrize("method", STDLIB_MSG_METHODS)
+    def test_stdlib_methods_support(self, method):
+        """
+        ReturnLogger implements methods of stdlib loggers.
+        """
+        v = getattr(ReturnLogger(), method)("hello")
+
+        assert "hello" == v
+
+
 class TestReturnLoggerFactory(object):
     def test_builds_returnloggers(self):
         f = ReturnLoggerFactory()
+
         assert isinstance(f(), ReturnLogger)
 
     def test_caches(self):
@@ -85,6 +131,7 @@ class TestReturnLoggerFactory(object):
         each call.
         """
         f = ReturnLoggerFactory()
+
         assert f() is f()
 
     def test_ignores_args(self):
