@@ -495,7 +495,7 @@ class TestProcessorFormatter(object):
 
         assert ("", "foo [in test_foreign_delegate]\n") == capsys.readouterr()
 
-    def test_clears_args(self, capsys, configure_for_pf):
+    def test_clears_args(self, configure_for_pf, capsys):
         """
         We render our log records before sending it back to logging.  Therefore
         we must clear `LogRecord.args` otherwise the user gets an
@@ -511,6 +511,19 @@ class TestProcessorFormatter(object):
             "hello world. [in test_clears_args]\n",
         ) == capsys.readouterr()
 
+    def test_log_dict(self, configure_for_pf, capsys):
+        """
+        Test that dicts can be logged with std library loggers.
+        """
+        configure_logging(None)
+
+        logging.getLogger().warning({"foo": "bar"})
+
+        assert (
+            "",
+            "{'foo': 'bar'} [in test_log_dict]\n",
+        ) == capsys.readouterr()
+
     def test_foreign_pre_chain(self, configure_for_pf, capsys):
         """
         If foreign_pre_chain is an iterable, it's used to pre-process
@@ -549,6 +562,28 @@ class TestProcessorFormatter(object):
             "e_chain_add_logger_name]\n",
         ) == capsys.readouterr()
 
+    def test_foreign_chain_can_pass_dictionaries_without_excepting(
+        self, configure_for_pf, capsys
+    ):
+        """
+        If a foreign logger passes a dictionary to a logging function,
+        check we correctly identify that it did not come from structlog.
+        """
+        configure_logging(None)
+        configure(
+            processors=[ProcessorFormatter.wrap_for_formatter],
+            logger_factory=LoggerFactory(),
+            wrapper_class=BoundLogger,
+        )
+
+        logging.getLogger().warning({"foo": "bar"})
+
+        assert (
+            "",
+            "{'foo': 'bar'} [in "
+            "test_foreign_chain_can_pass_dictionaries_without_excepting]\n",
+        ) == capsys.readouterr()
+
     def test_foreign_pre_chain_gets_exc_info(self, configure_for_pf, capsys):
         """
         If non-structlog record contains exc_info, foreign_pre_chain functions
